package yrpg.fi.cas.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.exception.KDBizException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

/**
 * 单据操作插件
 */
public class CheckBeforeNotifyHandlerOpPlugin extends AbstractOperationServicePlugIn implements Plugin {

    public void onPreparePropertys(PreparePropertysEventArgs e){
        e.getFieldKeys().add("billstatus");
        e.getFieldKeys().add("yrpg_paystatus");
        e.getFieldKeys().add("entry.e_sourcebillid");
        e.getFieldKeys().add("entry.e_sourcebillentryid");
        e.getFieldKeys().add("billno");
        e.getFieldKeys().add("sourcebilltype");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);

        DynamicObject[] bills = e.getDataEntities();
        for(DynamicObject bill:bills){
            //单据编号
            String billno=bill.getString("billno");

            //支付状态为支付失败、退票的付款处理单
            if(!"3".equals(bill.getString("yrpg_paystatus"))&&!"5".equals(bill.getString("yrpg_paystatus"))){
                throw new KDBizException("单据"+billno+"当前支付状态不为支付失败/退票，不可以进行通知经办人操作");
            }else{
                DynamicObjectCollection entrys=bill.getDynamicObjectCollection("entry");
                if(entrys.size()>0){
                    DynamicObject entry=entrys.get(0);
                    long sourcebillid=entry.getLong("e_sourcebillid");//来源单据id
                    long sourcebillentryid=entry.getLong("e_sourcebillentryid");//来源单据分录号
                    String sourcebilltype=bill.getString("sourcebilltype");//源单类型

                    if(sourcebillid!=0L&&sourcebillentryid!=0L&&!"".equals(sourcebilltype)){
                        if(!"er_checkingpaybill".equals(sourcebilltype)){ //商旅付款申请不做处理
                            String entrymsg=getEntryMsg(sourcebilltype);
                            if("".equals(entrymsg)){
                                throw new KDBizException("单据"+billno+"源单类型异常,不可以进行通知经办人操作");
                            }
                        }
                    }else{
                        throw new KDBizException("单据"+billno+"上游单据信息异常，不可以进行通知经办人操作");
                    }
                }
            }
        }
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);

        DynamicObject[] bills = e.getDataEntities();
        //回写付款处理单据上游报销单
        for(DynamicObject bill:bills){
            DynamicObjectCollection entrys=bill.getDynamicObjectCollection("entry");
            if(entrys.size()>0){
                DynamicObject entry=entrys.get(0);
                long sourcebillid=entry.getLong("e_sourcebillid");//来源单据id
                long sourcebillentryid=entry.getLong("e_sourcebillentryid");//来源单据分录号
                String sourcebilltype=bill.getString("sourcebilltype");//源单类型

                if(!"er_checkingpaybill".equals(sourcebilltype)){
                    String entrymsg=getEntryMsg(sourcebilltype);
                    DynamicObject sourcebill=BusinessDataServiceHelper.loadSingle(sourcebillid,sourcebilltype);

                    DynamicObjectCollection sourceentrys=sourcebill.getDynamicObjectCollection(entrymsg);
                    if(sourceentrys.size()>0){
                        for(DynamicObject sourceentry:sourceentrys){
                            if(sourcebillentryid==sourceentry.getLong("id")){
                                sourceentry.set("yrpg_chargebacks",true);
                            }
                        }
                    }

                    SaveServiceHelper.update(sourcebill);
                }
            }
        }
    }

    private String getEntryMsg(String sourcebilltype){
        switch(sourcebilltype)
        {
            case "er_dailyreimbursebill": //费用报销单
            case "er_publicreimbursebill":  //对公报销单
            case "er_tripreimbursebill": //差旅报销单
            case "er_dailyloanbill": //借款单
            case "er_prepaybill": //预付单
                return "accountentry";
            default :
                return "";
        }
    }
}